/***
*
*  Copyright (c) 1999, Valve LLC. All rights reserved.
*
*  This product contains software technology licensed from Id 
*  Software, Inc. ("Id Technology").  Id Technology (c) 1996 Id Software, Inc. 
*  All Rights Reserved.
*
*   Use, distribution, and modification of this source code and/or resulting
*   object code is restricted to non-commercial enhancements to products from
*   Valve LLC.  All other use, distribution, or modification is prohibited
*   without written permission from Valve LLC.
*
****/

#include <ctype.h>
#include <stdio.h>

#include <extdll.h>
#include <meta_api.h>		// of course

#include "moreutil.h"
#include "studio.h"

int gmsgTextMsg = 0;
int gmsgSayText = 0;
int gmsgShowMenu = 0;

edict_t *UTIL_MyFindEntityByClassname( edict_t *pentStart, const char *szName )
{
   return UTIL_MyFindEntityByString( pentStart, "classname", szName );
}

void UTIL_ParticleEffect( const Vector &vecOrigin, const Vector &vecDirection, ULONG ulColor, ULONG ulCount )
{
        PARTICLE_EFFECT( vecOrigin, vecDirection, (float)ulColor, (float)ulCount );
                }

void UTIL_ClientPrintAll( int msg_dest, const char *msg_name, const char *param1, const char *param2, const char *param3, const char *param4 )
{
   if (gmsgTextMsg==0)
     gmsgTextMsg = REG_USER_MSG( "TextMsg", -1);

	MESSAGE_BEGIN( MSG_ALL, gmsgTextMsg );
		WRITE_BYTE( msg_dest );
		WRITE_STRING( msg_name );

		if ( param1 )
			WRITE_STRING( param1 );
		if ( param2 )
			WRITE_STRING( param2 );
		if ( param3 )
			WRITE_STRING( param3 );
		if ( param4 )
			WRITE_STRING( param4 );

	MESSAGE_END();
}


void ClientPrint( edict_t *pEntity, int msg_dest, const char *msg_name)
{
   if (gmsgTextMsg == 0)
      gmsgTextMsg = REG_USER_MSG( "TextMsg", -1 );

   (*g_engfuncs.pfnMessageBegin)( MSG_ONE, gmsgTextMsg, NULL, pEntity );

   (*g_engfuncs.pfnWriteByte)( msg_dest );
   (*g_engfuncs.pfnWriteString)( msg_name );
   (*g_engfuncs.pfnMessageEnd)();
}

extern edict_t *listenserver_edict;

void UTIL_FakeMsg( int color, char *message )
{
   char  text[128];
   char *pc;
   edict_t *client;

if (strlen(message)>125)
  message[126]=0;
 

   // make sure the text has content
   for ( pc = message; pc != NULL && *pc != 0; pc++ )
   {
      if ( isprint( *pc ) && !isspace( *pc ) )
      {
         pc = NULL;   // we've found an alphanumeric character,  so text is valid
         break;
      }
   }

   if ( pc != NULL )
      return;  // no character found, so say nothing

      sprintf( text, "%c%s\n", color, message);

   // loop through all players
   // Start with the first player.
   // This may return the world in single player if the client types something between levels or during spawn
   // so check it, or it will infinite loop

   if (gmsgSayText == 0)
      gmsgSayText = REG_USER_MSG( "SayText", -1 );

   client = NULL;
   while ( ((client = UTIL_MyFindEntityByClassname( client, "player" )) != NULL) &&
           (!FNullEnt(client)) ) 
   {
//      if ( client == pEntity )  // skip sender of message
//         continue;

      (*g_engfuncs.pfnMessageBegin)( MSG_ONE, gmsgSayText, NULL, client );
         (*g_engfuncs.pfnWriteByte)( ENTINDEX(listenserver_edict) );
//         if (mod_id == FRONTLINE_DLL)
//            pfnWriteShort(0);
         (*g_engfuncs.pfnWriteString)( text );
      (*g_engfuncs.pfnMessageEnd)();
   }

   // echo to server console
   g_engfuncs.pfnServerPrint( text );
}

void UTIL_FakeMsgOne( int color, char *message, edict_t *client )
{
   char  text[128];
   char *pc;

if (strlen(message)>125)
  message[126]=0;

   // make sure the text has content
   for ( pc = message; pc != NULL && *pc != 0; pc++ )
   {
      if ( isprint( *pc ) && !isspace( *pc ) )
      {
         pc = NULL;   // we've found an alphanumeric character,  so text is valid
         break;
      }
   }

   if ( pc != NULL )
      return;  // no character found, so say nothing

      sprintf( text, "%c%s\n", color, message);

   // This may return the world in single player if the client types something between levels or during spawn
   // so check it, or it will infinite loop

   if (gmsgSayText == 0)
      gmsgSayText = REG_USER_MSG( "SayText", -1 );

      (*g_engfuncs.pfnMessageBegin)( MSG_ONE, gmsgSayText, NULL, client );
         (*g_engfuncs.pfnWriteByte)( ENTINDEX(listenserver_edict) );
//         if (mod_id == FRONTLINE_DLL)
//            pfnWriteShort(0);
         (*g_engfuncs.pfnWriteString)( text );
      (*g_engfuncs.pfnMessageEnd)();

//   UTIL_LogPrintf("Console Message: \"%s\" \"%s\"\n", STRING(client->v.netname), text+1);
}

void UTIL_MyHudMessage( edict_t *client, const hudtextparms_t &textparms, const char *pMessage )
{

if (!client)
  return;

	MESSAGE_BEGIN( MSG_ONE, SVC_TEMPENTITY, NULL, client );
		WRITE_BYTE( TE_TEXTMESSAGE );
		WRITE_BYTE( textparms.channel & 0xFF );

		WRITE_SHORT( FixedSigned16( textparms.x, 1<<13 ) );
		WRITE_SHORT( FixedSigned16( textparms.y, 1<<13 ) );
		WRITE_BYTE( textparms.effect );

		WRITE_BYTE( textparms.r1 );
		WRITE_BYTE( textparms.g1 );
		WRITE_BYTE( textparms.b1 );
		WRITE_BYTE( textparms.a1 );

		WRITE_BYTE( textparms.r2 );
		WRITE_BYTE( textparms.g2 );
		WRITE_BYTE( textparms.b2 );
		WRITE_BYTE( textparms.a2 );

		WRITE_SHORT( FixedUnsigned16( textparms.fadeinTime, 1<<8 ) );
		WRITE_SHORT( FixedUnsigned16( textparms.fadeoutTime, 1<<8 ) );
		WRITE_SHORT( FixedUnsigned16( textparms.holdTime, 1<<8 ) );

		if ( textparms.effect == 2 )
			WRITE_SHORT( FixedUnsigned16( textparms.fxTime, 1<<8 ) );
		
		if ( strlen( pMessage ) < 512 )
		{
			WRITE_STRING( pMessage );
		}
		else
		{
			char tmp[512];
			strncpy( tmp, pMessage, 511 );
			tmp[511] = 0;
			WRITE_STRING( tmp );
		}
	MESSAGE_END();
}

void UTIL_HudMessageAll( const hudtextparms_t &textparms, const char *pMessage )
{
        edict_t *client;
  
  client=NULL;
   while ( ((client = UTIL_MyFindEntityByClassname( client, "player" )) != NULL) &&
           (!FNullEnt(client)) ) 
   {
			UTIL_MyHudMessage( client, textparms, pMessage );
	}
}


//static 
short FixedSigned16( float value, float scale )
{
        int output;

        output = (int)(value * scale);

        if ( output > 32767 )
                output = 32767;

        if ( output < -32768 )
                output = -32768;

        return (short)output;
}

//static 
unsigned short FixedUnsigned16( float value, float scale )
{
        int output;

        output = (int)(value * scale);
        if ( output < 0 )
                output = 0;
        if ( output > 0xFFFF )
                output = 0xFFFF;

        return (unsigned short)output;
}

BOOL UTIL_IsVisible(edict_t *SourceEnt, edict_t *TargetEnt) {
TraceResult tr;
Vector vec1;
Vector vec2;
 
        // don't look through water
        if ((SourceEnt->v.waterlevel != 3 && TargetEnt->v.waterlevel == 3)
                || (SourceEnt->v.waterlevel == 3 && TargetEnt->v.waterlevel == 0))
                return FALSE;

vec1=SourceEnt->v.origin;
vec2=TargetEnt->v.origin;

UTIL_TraceLine(vec1, vec2, ignore_monsters, ignore_glass, SourceEnt, &tr);
 
if (tr.flFraction != 1.0)
 return FALSE;
else
 return TRUE;
}

edict_t *UTIL_MyFindEntityInSphere( edict_t *pentStart, const Vector &vecCenter, float flRadius )
{
   edict_t  *pentEntity;

   pentEntity = FIND_ENTITY_IN_SPHERE( pentStart, vecCenter, flRadius);

   if (!FNullEnt(pentEntity))
      return pentEntity;

   return NULL;
}

int UTIL_IsValidTarget(edict_t *pentMe, edict_t *pentTarg) {
vec_t zdiff;

  zdiff=(pentMe->v.origin).z-(pentTarg->v.origin).z;
  if (zdiff<0)
   zdiff = -zdiff;
  if (
     (GETPLAYERUSERID(pentTarg)!=-1) &&
     (pentTarg->v.deadflag == DEAD_NO) &&
     (zdiff<100) &&
     (UTIL_IsVisible(pentMe, pentTarg)) &&
     (!FBitSet( pentTarg->v.flags, FL_NOTARGET))     
     )
    return(1);

return(0);
}

edict_t *UTIL_FindPlayerVisInSphere( edict_t *pentOrig, const Vector &vecCenter, float flRadius )
{
   edict_t  *pentEntity;
   
  pentEntity=NULL;    
 while ((pentEntity = FIND_ENTITY_IN_SPHERE( pentEntity, vecCenter, flRadius))!=NULL)
  {
   if (FNullEnt(pentEntity))
      return NULL;
  if (UTIL_IsValidTarget(pentOrig, pentEntity))
      return(pentEntity);
  }

   return NULL;
}


edict_t *UTIL_MyFindEntityByString( edict_t *pentStart, const char *szKeyword, const char *szValue )
{
   edict_t *pentEntity;

   pentEntity = FIND_ENTITY_BY_STRING( pentStart, szKeyword, szValue );

   if (!FNullEnt(pentEntity))
      return pentEntity;
   return NULL;
}
void UTIL_EmitAmbientSound( edict_t *entity, const Vector &vecOrigin, const char *samp, float vol, float attenuation, int fFlags, int pitch )
{
	float rgfl[3];
	vecOrigin.CopyToArray(rgfl);

		EMIT_AMBIENT_SOUND(entity, rgfl, samp, vol, attenuation, fFlags, pitch);
}

Vector UTIL_VecToAngles( const Vector &vec )
{
	float rgflVecOut[3];
	VEC_TO_ANGLES(vec, rgflVecOut);
	return Vector(rgflVecOut);
}

void UTIL_TraceLine( const Vector &vecStart, const Vector &vecEnd, IGNORE_MONSTERS igmon, IGNORE_GLASS ignoreGlass, edict_t *pentIgnore, TraceResult *ptr )
{
	TRACE_LINE( vecStart, vecEnd, (igmon == ignore_monsters ? TRUE : FALSE) | (ignoreGlass?0x100:0), pentIgnore, ptr );
}

int UTIL_MySetActivity (edict_t *targ, Activity NewActivity) {
        int     iSequence;
        void *pmodel = GET_MODEL_PTR( targ );
        entvars_t *ev;
        
        ev=&(targ->v);
        iSequence = UTIL_LookupActivity ( pmodel, ev, NewActivity );

        if ( iSequence > -1)
        {

                targ->v.sequence           = iSequence;   
        targ->v.animtime = gpGlobals->time;
        targ->v.framerate = 1.0;
        targ->v.yaw_speed=100;
        }

return(1);
}

int UTIL_LookupActivity( void *pmodel, entvars_t *pev, int activity )
{
        studiohdr_t *pstudiohdr;

        pstudiohdr = (studiohdr_t *)pmodel;
        if (! pstudiohdr)
                return 0;

        mstudioseqdesc_t        *pseqdesc;

        pseqdesc = (mstudioseqdesc_t *)((byte *)pstudiohdr + pstudiohdr->seqindex);

        int weighttotal = 0;
        int seq = -1;
        for (int i = 0; i < pstudiohdr->numseq; i++)
        {
                if (pseqdesc[i].activity == activity)
                {
                        weighttotal += pseqdesc[i].actweight;
                        if (!weighttotal || RANDOM_LONG(0,weighttotal-1) < pseqdesc[i].actweight)
                                seq = i;
                }
        }

return seq;

}